<?php declare(strict_types=1);

namespace ConnectPayPayment\Components\Client\Hydrator\Request\InitiatePayment;

use ConnectPayPayment\Components\Client\Request\InitiatePaymentRequest;
use ConnectPayPayment\Components\Service\ConfigService\ConfigServiceInterface;
use ConnectPayPayment\Components\Struct\Configuration;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\System\Locale\LocaleEntity;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Component\Routing\RouterInterface;

class InitiatePaymentRequestHydrator implements InitiatePaymentRequestHydratorInterface
{
    private RouterInterface $router;
    private EntityRepositoryInterface $customerRepository;
    private EntityRepositoryInterface $localeRepository;

    /**
     * @param RouterInterface $router
     * @param EntityRepositoryInterface $customerRepository
     * @param EntityRepositoryInterface $localeRepository
     */
    public function __construct(
        RouterInterface $router,
        EntityRepositoryInterface $customerRepository,
        EntityRepositoryInterface $localeRepository
    ) {
        $this->router = $router;
        $this->customerRepository = $customerRepository;
        $this->localeRepository = $localeRepository;
    }

    /**
     * {@inheritdoc}
     */
    public function hydrate(
        AsyncPaymentTransactionStruct $transaction,
        Configuration $configuration,
        SalesChannelContext $context
    ): InitiatePaymentRequest {

        $order = $transaction->getOrder();

        $criteria = new Criteria([$order->getOrderCustomer()->getCustomerID()]);
        $criteria->addAssociation('defaultShippingAddress.country');
        $customer = $this->customerRepository->search($criteria, $context->getContext())->first();

        $criteria = new Criteria([$order->getLanguage()->getLocaleId()]);
        $locale = $this->localeRepository->search($criteria, $context->getContext())->first();

        $request = new InitiatePaymentRequest($configuration);

        $request->assign([
            'customer' => $customer,
            'transactionId' => $transaction->getOrderTransaction()->getId(),
            'salesChannelContextId' => $context->getSalesChannelId(),
            'brandId' => $configuration->get(ConfigServiceInterface::BRAND_ID),
            'paymentMethod' => key(self::PAYMENT_METHOD), // right now supported only PIS
            'paymentAmount' => $order->getPrice()->getTotalPrice(),
            'paymentCurrency' => $order->getCurrency()->getIsoCode(),
            'ipAddress' => $this->getIp(),
            'language' => $this->getMerchantLanguage($locale, $configuration),
            'providerCountryCode' => $configuration->get(ConfigServiceInterface::PROVIDER_COUNTRY_CODE) ?: '',
            'providerId' => trim(strip_tags($_COOKIE['bank_provider'] ?? '')),
            'notificationUrl' => $this->router->generate(
                'frontend.connect_pay.callback',
                [],
                UrlGeneratorInterface::ABSOLUTE_URL
            ),
            'redirectUrl' => $this->router->generate(
                'frontend.checkout.finish.page',
                ['orderId' => $order->getId()],
                UrlGeneratorInterface::ABSOLUTE_URL
            ),
        ]);
        
        return $request;
    }

    /**
     * 127.0.0.1 doesn't work, generate random IP for local testing
     *
     * @return string
     */
    private function getIp(): string
    {
        return !in_array($_SERVER['REMOTE_ADDR'], ['127.0.0.1', '::1'])
            ? $_SERVER['REMOTE_ADDR']
            : long2ip(rand(0, 4294967295));
    }

    /**
     * Define payment widget UI language
     *
     * @param LocaleEntity $locale
     * @param Configuration $configuration
     * @return string
     */
    private function getMerchantLanguage(LocaleEntity $locale, Configuration $configuration): string
    {
        if (in_array($locale->getCode(), self::AVAILABLE_MERCHANT_LANGUAGE)) {
            return $locale->getCode();
        }

        if ($configuration->has(ConfigServiceInterface::MERCHANT_LANGUAGE)) {
            return $configuration->get(ConfigServiceInterface::MERCHANT_LANGUAGE);
        }

        return self::DEFAULT_MERCHANT_LANGUAGE;
    }
}
