<?php declare(strict_types=1);

namespace ConnectPayPayment\Components\Client\Request;

use Shopware\Core\Checkout\Customer\CustomerEntity;
use Symfony\Component\HttpFoundation\Request;
use ConnectPayPayment\Components\Client\Request\Struct;

class InitiatePaymentRequest extends Struct implements RequestInterface
{
    protected string $method = Request::METHOD_POST;
    protected string $endpoint = '/merchant/payments';
    protected string $brandId;
    protected string $redirectUrl;
    protected string $notificationUrl;
    protected string $paymentMethod;
    protected string $paymentCurrency;
    protected string $language;
    protected string $providerCountryCode;
    protected string $providerId;
    protected string $salesChannelContextId;
    protected string $transactionId;
    protected string $ipAddress;
    protected float $paymentAmount;

    protected CustomerEntity $customer;

    public function getMethod(): string
    {
        return $this->method;
    }

    public function getEndpoint(): string
    {
        return $this->getBaseUrl().$this->endpoint;
    }

    public function getHeaders(): array
    {
        return [
            'Authorization' => $this->getBasicAuth(),
            'Content-Type' => 'application/json',
        ];
    }

    public function getPostData(): array
    {
        return ['body' => json_encode($this->jsonSerialize())];
    }

    public function jsonSerialize(): array
    {
        $data = ['merchant' => [
                'brandId' => $this->getBrandId(),
                'redirectUrl' => $this->getRedirectUrl(),
                'notificationUrl' => $this->getNotificationUrl(),
                'language' => $this->getLanguage(),
            ],
            'payment' => [
                'paymentMethod' => $this->getPaymentMethod(),
                'providerCountryCode' => $this->getProviderCountryCode(),
                "instructedAmount" => [
                    'amount' => number_format($this->getPaymentAmount(), 2, '.', ''),
                    'currency' => $this->getPaymentCurrency(),
                ],
                'remittanceInformationUnstructured' => $this->getSalesChannelContextId(),
            ],
            'identifiers' => [
                'merchantReference' => $this->getTransactionId(),
                'endToEndId' => "NOTPROVIDED",  //hardcoded value
            ],
            'consumer' => [
                'firstName' => $this->getCustomer()->getFirstName(),
                'lastName' => $this->getCustomer()->getLastName(),
                'ipAddress' => $this->getIpAddress(),
                'email' => $this->getCustomer()->getEmail(),
            ],
        ];
        
        if ($this->getProviderId()) {
            $data['payment']['providerId'] = $this->getProviderId();
        }
        
        return $data;

    }

    public function getBrandId(): string
    {
        return $this->brandId;
    }

    public function getRedirectUrl(): string
    {
        return $this->redirectUrl;
    }

    public function getNotificationUrl(): string
    {
        return $this->notificationUrl;
    }

    public function getLanguage(): string
    {
        return $this->language;
    }

    public function getPaymentMethod(): string
    {
        return $this->paymentMethod;
    }

    public function getProviderCountryCode(): string
    {
        return $this->providerCountryCode;
    }

    public function getProviderId(): string
    {
        return $this->providerId;
    }

    public function getPaymentAmount(): float
    {
        return $this->paymentAmount;
    }

    public function getPaymentCurrency(): string
    {
        return $this->paymentCurrency;
    }

    public function getSalesChannelContextId(): string
    {
        return $this->salesChannelContextId;
    }

    public function getTransactionId(): string
    {
        return $this->transactionId;
    }

    public function getCustomer(): CustomerEntity
    {
        return $this->customer;
    }

    private function getIpAddress(): string
    {
        return $this->ipAddress;
    }
}
