<?php declare(strict_types=1);

namespace ConnectPayPayment\Components\PaymentHandler;

use ConnectPayPayment\Components\Client\ClientInterface;
use ConnectPayPayment\Components\Client\Hydrator\Request\GetPaymentDetails\GetPaymentDetailsRequestHydratorInterface;
use ConnectPayPayment\Components\Client\Hydrator\Request\InitiatePayment\InitiatePaymentRequestHydratorInterface;
use ConnectPayPayment\Components\Client\Hydrator\Response\GetPaymentDetails\GetPaymentDetailsResponseHydratorInterface;
use ConnectPayPayment\Components\Client\Response\GetPaymentDetailsResponse;
use ConnectPayPayment\Components\Client\Struct\PaymentDetailsStruct;
use ConnectPayPayment\Components\Helper\OrderTransaction\OrderTransactionHelperInterface;
use ConnectPayPayment\Components\Service\ConfigService\ConfigServiceInterface;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Cart\PaymentHandler\AsynchronousPaymentHandlerInterface;
use Shopware\Core\Checkout\Payment\Exception\AsyncPaymentFinalizeException;
use Shopware\Core\Checkout\Payment\Exception\AsyncPaymentProcessException;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Core\System\StateMachine\Exception\IllegalTransitionException;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Contracts\Translation\TranslatorInterface;

class AsynchronousPaymentHandler extends AbstractPaymentHandler implements AsynchronousPaymentHandlerInterface
{
    private OrderTransactionStateHandler $transactionStateHandler;
    private InitiatePaymentRequestHydratorInterface $initiatePaymentHydrator;
    private GetPaymentDetailsRequestHydratorInterface $paymentDetailsRequestHydrator;
    private GetPaymentDetailsResponseHydratorInterface $paymentDetailsResponseHydrator;
    private ClientInterface $client;
    private TranslatorInterface $translator;
    private ConfigServiceInterface $configService;
    private OrderTransactionHelperInterface $orderTransactionHelper;

    /**
     * @param OrderTransactionStateHandler $transactionStateHandler
     * @param InitiatePaymentRequestHydratorInterface $initiatePaymentHydrator
     * @param GetPaymentDetailsRequestHydratorInterface $paymentDetailsRequestHydrator
     * @param GetPaymentDetailsResponseHydratorInterface $paymentDetailsResponseHydrator
     * @param ClientInterface $client
     * @param TranslatorInterface $translator
     * @param RequestStack $requestStack
     * @param ConfigServiceInterface $configService
     * @param OrderTransactionHelperInterface $orderTransactionHelper
     */
    public function __construct(
        OrderTransactionStateHandler $transactionStateHandler,
        InitiatePaymentRequestHydratorInterface $initiatePaymentHydrator,
        GetPaymentDetailsRequestHydratorInterface $paymentDetailsRequestHydrator,
        GetPaymentDetailsResponseHydratorInterface $paymentDetailsResponseHydrator,
        ClientInterface $client,
        TranslatorInterface $translator,
        RequestStack $requestStack,
        ConfigServiceInterface $configService,
        OrderTransactionHelperInterface $orderTransactionHelper
    ) {
        $this->transactionStateHandler = $transactionStateHandler;
        $this->initiatePaymentHydrator = $initiatePaymentHydrator;
        $this->paymentDetailsRequestHydrator = $paymentDetailsRequestHydrator;
        $this->paymentDetailsResponseHydrator = $paymentDetailsResponseHydrator;
        $this->client = $client;
        $this->translator = $translator;
        $this->requestStack = $requestStack;
        $this->configService = $configService;
        $this->orderTransactionHelper = $orderTransactionHelper;
    }

    /**
     * {@inheritdoc}
     */
    public function pay(
        AsyncPaymentTransactionStruct $transaction,
        RequestDataBag $dataBag,
        SalesChannelContext $salesChannelContext
    ): RedirectResponse {

        $request = $this->initiatePaymentHydrator->hydrate(
            $transaction,
            $this->configService->get(),
            $salesChannelContext
        );
        $response = $this->client->request($request);

        if (Response::HTTP_CREATED !== $response->getHttpStatus()) {
            throw new AsyncPaymentProcessException(
                $transaction->getOrderTransaction()->getId(),
                $this->translator->trans('connectPay.errorMessages.invalidRequest')
            );
        }

        $this->orderTransactionHelper->savePaymentId(
            $response->getResponse()['payment']['paymentId'],
            $transaction->getOrderTransaction(),
            $salesChannelContext
        );

        return new RedirectResponse($response->getResponse()['payment']['redirectUrl']);
    }

    /**
     * {@inheritdoc}
     */
    public function checkPaymentStatus(OrderTransactionEntity $orderTransaction): ?GetPaymentDetailsResponse
    {
        $paymentId = $orderTransaction->getCustomFields()['paymentId'] ?? null;
        if (!($paymentId)) {
            return null;
        }

        $request = $this->paymentDetailsRequestHydrator->hydrate(
            $paymentId,
            $this->configService->get(),
        );
        $response = $this->client->request($request);

        if (Response::HTTP_OK !== $response->getHttpStatus()) {
            return null;
        }

        $response = $this->paymentDetailsResponseHydrator->hydrate($response);
        if ($response->getIsTerminal() && $response->getTransaction() && $response->getSalesChannelContext()) {
            $this->finalize(
                new PaymentDetailsStruct(
                    $response->getTransaction(),
                    $response->getTransaction()->getOrder(),
                    '',
                    $response->getCode()
                ),
                new Request(),
                $response->getSalesChannelContext()
            );
        }

        return $response;
    }

    /**
     * {@inheritdoc}
     */
    public function finalize(
        AsyncPaymentTransactionStruct $transaction,
        Request $request,
        SalesChannelContext $salesChannelContext
    ): void {
        $transactionId = $transaction->getOrderTransaction()->getId();

        try {
            self::APPROVED === $transaction->getCode()
                ? $this->transactionStateHandler->paid($transactionId, $salesChannelContext->getContext())
                : $this->transactionStateHandler->fail($transactionId, $salesChannelContext->getContext());
        } catch (IllegalTransitionException $paymentProcessException) {
            throw new AsyncPaymentFinalizeException($transactionId, '');
        }
    }
}
