<?php declare(strict_types=1);

namespace ConnectPayPayment\Installer;

use ConnectPayPayment\ConnectPayPayment;
use Shopware\Core\Content\Media\File\FileSaver;
use Shopware\Core\Content\Media\File\MediaFile;
use Shopware\Core\Defaults;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Plugin\Util\PluginIdProvider;
use Shopware\Core\Framework\Uuid\Uuid;

class PaymentInstaller implements InstallerInterface
{
    public const MEDIA_PATH = __DIR__.'/../Resources/config/plugin.png';

    public const PAYMENT_METHODS = [
        ConnectPayPayment::PAYMENT_HANDLER_IDENTIFIER => [
            'en-GB' => [
                'name' => 'ConnectPay',
            ],
            'de-DE' => [
                'name' => 'ConnectPay',
            ],
            'nl-NL' => [
                'name' => 'ConnectPay',
            ],
            'sv-SE' => [
                'name' => 'ConnectPay',
            ],
            'lt-LT' => [
                'name' => 'ConnectPay',
            ],
            'fi-FI' => [
                'name' => 'ConnectPay',
            ],
        ],
    ];

    private Context $context;
    private PluginIdProvider $pluginIdProvider;
    private EntityRepositoryInterface $paymentMethodRepository;
    private EntityRepositoryInterface $languageRepository;
    private EntityRepositoryInterface $mediaRepository;
    private FileSaver $fileSaver;

    /**
     * @param Context $context
     * @param PluginIdProvider $pluginIdProvider
     * @param EntityRepositoryInterface $paymentMethodRepository
     * @param EntityRepositoryInterface $languageRepository
     * @param EntityRepositoryInterface $mediaRepository
     * @param FileSaver $fileSaver
     */
    public function __construct(
        Context $context,
        PluginIdProvider $pluginIdProvider,
        EntityRepositoryInterface $paymentMethodRepository,
        EntityRepositoryInterface $languageRepository,
        EntityRepositoryInterface $mediaRepository,
        FileSaver $fileSaver
    ) {
        $this->context = $context;
        $this->pluginIdProvider = $pluginIdProvider;
        $this->paymentMethodRepository = $paymentMethodRepository;
        $this->languageRepository = $languageRepository;
        $this->mediaRepository = $mediaRepository;
        $this->fileSaver = $fileSaver;
    }

    /**
     * {@inheritDoc}
     */
    public function install(): void
    {
        $this->upsertPaymentMethods();
    }

    /**
     * Create payment methods
     */
    private function upsertPaymentMethods(): void
    {
        $defaultLocaleCode = $this->getSystemDefaultLocaleCode();
        foreach (self::PAYMENT_METHODS as $paymentMethodHandlerIdentifier => $translations) {
            $paymentMethodId = $this->getPaymentMethodIdForHandlerIdentifier($paymentMethodHandlerIdentifier);
            if ($defaultLocaleCode && !isset($translations[$defaultLocaleCode])) {
                $translations[$defaultLocaleCode] = $translations['en-GB'];
            }

            $this->paymentMethodRepository->upsert([
                [
                    'id' => $paymentMethodId,
                    'handlerIdentifier' => $paymentMethodHandlerIdentifier,
                    'pluginId' => $this->getPluginId(),
                    'mediaId' => $this->getMediaId($translations[$defaultLocaleCode]['name']),
                    'translations' => $translations,
                ],
            ], $this->context);
        }
    }

    /**
     * Retrieve default locale
     *
     * @return string|null
     */
    private function getSystemDefaultLocaleCode(): ?string
    {
        $criteria = new Criteria([Defaults::LANGUAGE_SYSTEM]);
        $criteria->addAssociation('locale');
        $systemDefaultLanguage = $this->languageRepository->search($criteria, $this->context)->first();

        $locale = $systemDefaultLanguage->getLocale();
        if (!$locale) {
            return null;
        }

        return $locale->getCode();
    }

    /**
     * Get payment methods by technical name
     *
     * @param string $paymentMethodHandlerIdentifier
     * @return string|null
     */
    private function getPaymentMethodIdForHandlerIdentifier(string $paymentMethodHandlerIdentifier): ?string
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('handlerIdentifier', $paymentMethodHandlerIdentifier));

        return $this->paymentMethodRepository
            ->searchIds($criteria, $this->context)
            ->firstId();
    }

    /**
     * Retrieve plugin id
     *
     * @return string
     */
    private function getPluginId(): string
    {
        return $this->pluginIdProvider->getPluginIdByBaseClass(
            ConnectPayPayment::class,
            $this->context
        );
    }

    /**
     * Create payment icon
     *
     * @param string $fileName
     * @return string|null
     */
    private function getMediaId(string $fileName): ?string
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('fileName', $fileName));
        $media = $this->mediaRepository->search($criteria, $this->context);

        if ($media->count() && $media->first() !== null) {
            return $media->first()->getId();
        }

        $mediaPath = self::MEDIA_PATH;
        if (!file_exists($mediaPath)) {
            return null;
        }

        $mediaFile = new MediaFile(
            $mediaPath,
            mime_content_type($mediaPath),
            pathinfo($mediaPath, PATHINFO_EXTENSION),
            filesize($mediaPath)
        );

        $mediaId = Uuid::randomHex();
        $mediaData = ['id' => $mediaId];
        $this->mediaRepository->create([$mediaData], $this->context);

        $this->fileSaver->persistFileToMedia(
            $mediaFile,
            $fileName,
            $mediaId,
            $this->context
        );

        return $mediaId;
    }

    /**
     * {@inheritDoc}
     */
    public function update(): void
    {
        $this->upsertPaymentMethods();
    }

    /**
     * {@inheritDoc}
     */
    public function activate(): void
    {
        $this->activatePaymentMethods();
    }

    /**
     * Set payment methods active
     */
    private function activatePaymentMethods(): void
    {
        foreach (self::PAYMENT_METHODS as $paymentMethodHandlerIdentifier => $translations) {
            $paymentMethodId = $this->getPaymentMethodIdForHandlerIdentifier($paymentMethodHandlerIdentifier);
            if (!$paymentMethodId) {
                continue;
            }

            $this->paymentMethodRepository->update([
                [
                    'id' => $paymentMethodId,
                    'active' => true,
                ],
            ], $this->context);
        }
    }

    /**
     * {@inheritDoc}
     */
    public function deactivate(): void
    {
        $this->deactivatePaymentMethods();
    }

    /**
     * Set payment methods inactive
     */
    private function deactivatePaymentMethods(): void
    {
        foreach (self::PAYMENT_METHODS as $paymentMethodHandlerIdentifier => $translations) {
            $paymentMethodId = $this->getPaymentMethodIdForHandlerIdentifier($paymentMethodHandlerIdentifier);
            if (!$paymentMethodId) {
                continue;
            }

            $this->paymentMethodRepository->update([
                [
                    'id' => $paymentMethodId,
                    'active' => false,
                ],
            ], $this->context);
        }
    }

    /**
     * {@inheritDoc}
     */
    public function uninstall(): void
    {
    }
}
